@@ -52,7 +52,6 @@ package com.openexchange.imap;
 import java.io.IOException;
 import java.io.Writer;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.EnumSet;
 import java.util.List;
 import java.util.Set;
@@ -67,6 +66,7 @@ import javax.mail.MessagingException;
 import javax.mail.MethodNotSupportedException;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.groupware.contexts.Context;
+import com.openexchange.imap.cache.MBoxEnabledCache;
 import com.openexchange.imap.cache.RootSubfolderCache;
 import com.openexchange.imap.config.IIMAPProperties;
 import com.openexchange.imap.config.IMAPConfig;
@@ -137,7 +137,7 @@ public final class IMAPDefaultFolderChecker {
         this.imapConfig = imapConfig;
         final int timeout = ((IIMAPProperties) imapConfig.getMailProperties()).getImapTimeout();
         if (timeout <= 0) {
-            final ConfigurationService confService = IMAPServiceRegistry.getServiceRegistry().getService(ConfigurationService.class);
+            final ConfigurationService confService = IMAPServiceRegistry.getService(ConfigurationService.class);
             if (null == confService) {
                 // Default of 2 minutes
                 maxRunningMillis = DEFAULT_MAX_RUNNING_MILLIS;
@@ -248,14 +248,13 @@ public final class IMAPDefaultFolderChecker {
                          */
                         inboxFolder.setSubscribed(true);
                     }
-                    final StringBuilder tmp = new StringBuilder(128);
                     /*
                      * Get prefix for default folder names, NOT fullnames!
                      */
                     final String prefix;
                     final char sep;
                     {
-                        final String[] sa = getDefaultFolderPrefix(inboxFolder, tmp);
+                        final String[] sa = getDefaultFolderPrefix(inboxFolder);
                         prefix = sa[0];
                         sep = sa[1].charAt(0);
                     }
@@ -263,16 +262,8 @@ public final class IMAPDefaultFolderChecker {
                      * Check for mbox
                      */
                     final int type;
-                    final boolean mboxEnabled;
-                    {
-                        final String param = MailSessionParameterNames.getParamMBox(accountId);
-                        Boolean mbox = (Boolean) session.getParameter(param);
-                        if (null == mbox) {
-                            mbox = Boolean.valueOf(!IMAPCommandsCollection.supportsFolderType(inboxFolder, FOLDER_TYPE, prefix));
-                            session.setParameter(param, mbox);
-                        }
-                        mboxEnabled = mbox.booleanValue();
-                    }
+                    final boolean mboxEnabled =
+                        MBoxEnabledCache.isMBoxEnabled(imapConfig.getImapServerSocketAddress(), inboxFolder, prefix);
                     if (mboxEnabled) {
                         type = IMAPFolder.HOLDS_MESSAGES;
                     } else {
@@ -284,10 +275,11 @@ public final class IMAPDefaultFolderChecker {
                     final boolean isSpamOptionEnabled;
                     final MailAccount mailAccount;
                     try {
-                        mailAccount = IMAPServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true).getMailAccount(
-                            accountId,
-                            session.getUserId(),
-                            session.getContextId());
+                        mailAccount =
+                            IMAPServiceRegistry.getService(MailAccountStorageService.class, true).getMailAccount(
+                                accountId,
+                                session.getUserId(),
+                                session.getContextId());
                         final UserSettingMail usm = UserSettingMailStorage.getInstance().getUserSettingMail(session.getUserId(), ctx);
                         isSpamOptionEnabled = usm.isSpamOptionEnabled();
                     } catch (final ServiceException e) {
@@ -298,23 +290,20 @@ public final class IMAPDefaultFolderChecker {
                     /*
                      * Check default folders
                      */
-                    final DefaultFolderNamesProvider defaultFolderNamesProvider = new DefaultFolderNamesProvider(
-                        accountId,
-                        session.getUserId(),
-                        session.getContextId());
-                    final String[] defaultFolderFullnames = defaultFolderNamesProvider.getDefaultFolderFullnames(
-                        mailAccount,
-                        isSpamOptionEnabled);
+                    final DefaultFolderNamesProvider defaultFolderNamesProvider =
+                        new DefaultFolderNamesProvider(accountId, session.getUserId(), session.getContextId());
+                    final String[] defaultFolderFullnames =
+                        defaultFolderNamesProvider.getDefaultFolderFullnames(mailAccount, isSpamOptionEnabled);
                     final String[] defaultFolderNames = defaultFolderNamesProvider.getDefaultFolderNames(mailAccount, isSpamOptionEnabled);
                     final SpamHandler spamHandler;
                     {
-                        spamHandler = isSpamOptionEnabled ? SpamHandlerRegistry.getSpamHandlerBySession(session, accountId) : NoSpamHandler.getInstance();
+                        spamHandler =
+                            isSpamOptionEnabled ? SpamHandlerRegistry.getSpamHandlerBySession(session, accountId) : NoSpamHandler.getInstance();
                     }
                     final CompletionService<Object> completionService;
                     try {
-                        completionService = new ExecutorCompletionService<Object>(IMAPServiceRegistry.getServiceRegistry().getService(
-                            TimerService.class,
-                            true).getExecutor());
+                        completionService =
+                            new ExecutorCompletionService<Object>(IMAPServiceRegistry.getService(TimerService.class, true).getExecutor());
                     } catch (final ServiceException e) {
                         throw new IMAPException(e);
                     }
@@ -332,8 +321,7 @@ public final class IMAPDefaultFolderChecker {
                                     defaultFolderNames[index],
                                     sep,
                                     type,
-                                    spamHandler.isUnsubscribeSpamFolders() ? 0 : -1,
-                                    tmp.toString());
+                                    spamHandler.isUnsubscribeSpamFolders() ? 0 : -1);
                                 count++;
                             } else if (LOG.isDebugEnabled()) {
                                 LOG.debug("Skipping check for " + defaultFolderNames[index] + " due to SpamHandler.isCreateConfirmedHam()=false");
@@ -348,14 +336,13 @@ public final class IMAPDefaultFolderChecker {
                                     defaultFolderNames[index],
                                     sep,
                                     type,
-                                    spamHandler.isUnsubscribeSpamFolders() ? 0 : -1,
-                                    tmp.toString());
+                                    spamHandler.isUnsubscribeSpamFolders() ? 0 : -1);
                                 count++;
                             } else if (LOG.isDebugEnabled()) {
                                 LOG.debug("Skipping check for " + defaultFolderNames[index] + " due to SpamHandler.isCreateConfirmedSpam()=false");
                             }
                         } else {
-                            submitFolderCheckTask(completionService, index, prefix, fullname, defaultFolderNames[index], sep, type, 1, tmp.toString());
+                            submitFolderCheckTask(completionService, index, prefix, fullname, defaultFolderNames[index], sep, type, 1);
                             count++;
                         }
                     }
@@ -402,15 +389,15 @@ public final class IMAPDefaultFolderChecker {
         }
     }
 
-    private void submitFolderCheckTask(final CompletionService<Object> completionService, final int index, final String prefix, final String fullname, final String defaultFolderName, final char sep, final int type, final int subscribe, final String tmp) {
+    private void submitFolderCheckTask(final CompletionService<Object> completionService, final int index, final String prefix, final String fullname, final String defaultFolderName, final char sep, final int type, final int subscribe) {
         completionService.submit(new AbstractCallable(imapConfig, session) {
 
             public Object call() throws MailException {
                 try {
                     if (null == fullname || 0 == fullname.length()) {
-                        setDefaultMailFolder(index, checkDefaultFolder(index, prefix, defaultFolderName, sep, type, subscribe, false, tmp));
+                        setDefaultMailFolder(index, checkDefaultFolder(index, prefix, defaultFolderName, sep, type, subscribe, false));
                     } else {
-                        setDefaultMailFolder(index, checkDefaultFolder(index, "", fullname, sep, type, subscribe, true, tmp));
+                        setDefaultMailFolder(index, checkDefaultFolder(index, "", fullname, sep, type, subscribe, true));
                     }
                     return null;
                 } catch (final MessagingException e) {
@@ -420,12 +407,13 @@ public final class IMAPDefaultFolderChecker {
         });
     }
 
-    private String[] getDefaultFolderPrefix(final IMAPFolder inboxFolder, final StringBuilder tmp) throws MessagingException, IMAPException {
+    private String[] getDefaultFolderPrefix(final IMAPFolder inboxFolder) throws MessagingException, IMAPException {
         /*
          * Check for NAMESPACE capability
          */
         final char sep;
         final String inboxfullName = inboxFolder.getFullName();
+        final StringBuilder prefix = new StringBuilder(16);
         if (imapConfig.getImapCapabilities().hasNamespace()) {
             /*
              * Perform the NAMESPACE command to detect the subfolder prefix. From rfc2342: Clients often attempt to create mailboxes for
@@ -449,10 +437,10 @@ public final class IMAPDefaultFolderChecker {
                      * Personal namespace folder allows subfolders and nested default folder are demanded, thus use INBOX as prefix although
                      * NAMESPACE signals to use no prefix.
                      */
-                    tmp.append(inboxfullName).append(sep);
+                    prefix.append(inboxfullName).append(sep);
                 }
             } else {
-                tmp.append(persPrefix).append(sep);
+                prefix.append(persPrefix).append(sep);
             }
         } else {
             /*
@@ -464,37 +452,32 @@ public final class IMAPDefaultFolderChecker {
             /*
              * Examine root folder if subfolders allowed
              */
-            final boolean rootInferiors = RootSubfolderCache.canCreateSubfolders(
-                (DefaultFolder) imapStore.getDefaultFolder(),
-                true,
-                session,
-                accountId).booleanValue();
+            final boolean rootInferiors =
+                RootSubfolderCache.canCreateSubfolders((DefaultFolder) imapStore.getDefaultFolder(), true, session, accountId).booleanValue();
             /*
              * Determine where to create default folders and store as a prefix for folder fullname
              */
             if (!inboxInferiors && rootInferiors) {
                 // Create folder beside INBOX folder
-                tmp.append("");
+                prefix.append("");
             } else if (inboxInferiors && !rootInferiors) {
                 // Create folder under INBOX folder
-                tmp.append(inboxfullName).append(sep);
+                prefix.append(inboxfullName).append(sep);
             } else if (inboxInferiors && rootInferiors) {
                 if (MailProperties.getInstance().isAllowNestedDefaultFolderOnAltNamespace()) {
                     /*
                      * Only allow default folder below INBOX if inferiors are permitted nested default folder are explicitly allowed
                      */
-                    tmp.append(inboxfullName).append(sep);
+                    prefix.append(inboxfullName).append(sep);
                 } else {
-                    tmp.append("");
+                    prefix.append("");
                 }
             } else {
                 // Cannot occur: No folders are allowed to be created, neither below INBOX nor below root folder
                 throw IMAPException.create(IMAPException.Code.NO_CREATE_ACCESS, imapConfig, session, "INBOX");
             }
         }
-        final String prefix = tmp.toString();
-        tmp.setLength(0);
-        return new String[] { prefix, String.valueOf(sep) };
+        return new String[] { prefix.toString(), String.valueOf(sep) };
     }
 
     /**
@@ -518,11 +501,11 @@ public final class IMAPDefaultFolderChecker {
     /**
      * Internally used by {@link IMAPDefaultFolderChecker}.
      */
-    String checkDefaultFolder(final int index, final String prefix, final String name, final char sep, final int type, final int subscribe, final boolean isFullname, String tmpStarter) throws MessagingException, IMAPException {
+    String checkDefaultFolder(final int index, final String prefix, final String name, final char sep, final int type, final int subscribe, final boolean isFullname) throws MessagingException, IMAPException {
         /*
          * Check default folder
          */
-        StringBuilder tmp = new StringBuilder(tmpStarter);
+        final StringBuilder tmp = new StringBuilder(32);
         final boolean checkSubscribed = true;
         final long st = System.currentTimeMillis();
         Folder f = imapStore.getFolder(tmp.append(prefix).append(name).toString());
@@ -604,9 +587,8 @@ public final class IMAPDefaultFolderChecker {
                             throw new MessagingException("Unexpected index: " + index);
                         }
                         try {
-                            final MailAccountStorageService storageService = IMAPServiceRegistry.getServiceRegistry().getService(
-                                MailAccountStorageService.class,
-                                true);
+                            final MailAccountStorageService storageService =
+                                IMAPServiceRegistry.getService(MailAccountStorageService.class, true);
                             storageService.updateMailAccount(
                                 mad,
                                 attributes,
