@@ -73,6 +73,7 @@ import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.groupware.contexts.impl.ContextStorage;
 import com.openexchange.imap.acl.ACLExtension;
 import com.openexchange.imap.acl.ACLExtensionFactory;
+import com.openexchange.imap.cache.MBoxEnabledCache;
 import com.openexchange.imap.cache.NamespaceFoldersCache;
 import com.openexchange.imap.cache.RightsCache;
 import com.openexchange.imap.cache.UserFlagsCache;
@@ -83,9 +84,9 @@ import com.openexchange.imap.converters.IMAPFolderConverter;
 import com.openexchange.imap.entity2acl.Entity2ACL;
 import com.openexchange.imap.entity2acl.Entity2ACLArgs;
 import com.openexchange.imap.entity2acl.Entity2ACLException;
+import com.openexchange.imap.entity2acl.UserGroupID;
 import com.openexchange.imap.util.IMAPSessionStorageAccess;
 import com.openexchange.mail.MailException;
-import com.openexchange.mail.MailSessionParameterNames;
 import com.openexchange.mail.api.MailFolderStorage;
 import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.dataobjects.MailFolder;
@@ -486,17 +487,11 @@ public final class IMAPFolderStorage extends MailFolderStorage {
              * Check if IMAP server is in MBox format; meaning folder either hold messages or subfolders but not both
              */
             final char separator = parent.getSeparator();
-            final boolean mboxEnabled;
-            {
-                final String param = MailSessionParameterNames.getParamMBox(accountId);
-                Boolean mbox = (Boolean) session.getParameter(param);
-                if (null == mbox) {
-                    mbox = Boolean.valueOf(!IMAPCommandsCollection.supportsFolderType(parent, FOLDER_TYPE, new StringBuilder(
-                        parent.getFullName()).append(separator).toString()));
-                    session.setParameter(param, mbox);
-                }
-                mboxEnabled = mbox.booleanValue();
-            }
+            final boolean mboxEnabled =
+                MBoxEnabledCache.isMBoxEnabled(
+                    imapConfig.getImapServerSocketAddress(),
+                    parent,
+                    new StringBuilder(parent.getFullName()).append(separator).toString());
             if (!checkFolderNameValidity(name, separator, mboxEnabled)) {
                 throw IMAPException.create(IMAPException.Code.INVALID_FOLDER_NAME, imapConfig, session, Character.valueOf(separator));
             }
@@ -506,7 +501,8 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                  */
                 createMe = (IMAPFolder) imapStore.getFolder(name);
             } else {
-                createMe = (IMAPFolder) imapStore.getFolder(new StringBuilder(parent.getFullName()).append(separator).append(name).toString());
+                createMe =
+                    (IMAPFolder) imapStore.getFolder(new StringBuilder(parent.getFullName()).append(separator).append(name).toString());
             }
             if (createMe.exists()) {
                 throw IMAPException.create(IMAPException.Code.DUPLICATE_FOLDER, imapConfig, session, createMe.getFullName());
@@ -570,7 +566,8 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                     final Entity2ACLArgs args = IMAPFolderConverter.getEntity2AclArgs(session, createMe, imapConfig);
                     final Map<String, ACL> m = acl2map(newACLs);
                     if (!equals(initialACLs, m, entity2ACL, args)) {
-                        if (!getACLExtension().canSetACL(createMe.myRights())) {
+                        final ACLExtension aclExtension = getACLExtension();
+                        if (!aclExtension.canSetACL(createMe.myRights())) {
                             throw IMAPException.create(
                                 IMAPException.Code.NO_ADMINISTER_ACCESS_ON_INITIAL,
                                 imapConfig,
@@ -579,7 +576,7 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                         }
                         boolean adminFound = false;
                         for (int i = 0; (i < newACLs.length) && !adminFound; i++) {
-                            if (getACLExtension().canSetACL(newACLs[i].getRights())) {
+                            if (aclExtension.canSetACL(newACLs[i].getRights())) {
                                 adminFound = true;
                             }
                         }
@@ -723,7 +720,8 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                 if (getChecker().isDefaultFolder(oldFullname)) {
                     throw IMAPException.create(IMAPException.Code.NO_DEFAULT_FOLDER_UPDATE, imapConfig, session, oldFullname);
                 }
-                IMAPFolder destFolder = ((IMAPFolder) (MailFolder.DEFAULT_FOLDER_ID.equals(newParent) ? imapStore.getDefaultFolder() : imapStore.getFolder(newParent)));
+                IMAPFolder destFolder =
+                    ((IMAPFolder) (MailFolder.DEFAULT_FOLDER_ID.equals(newParent) ? imapStore.getDefaultFolder() : imapStore.getFolder(newParent)));
                 if (!destFolder.exists()) {
                     destFolder = checkForNamespaceFolder(newParent);
                     if (null == destFolder) {
@@ -765,17 +763,9 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                         }
                     }
                 }
-                final boolean mboxEnabled;
-                {
-                    final String param = MailSessionParameterNames.getParamMBox(accountId);
-                    Boolean mbox = (Boolean) session.getParameter(param);
-                    if (null == mbox) {
-                        mbox = Boolean.valueOf(!IMAPCommandsCollection.supportsFolderType(destFolder, FOLDER_TYPE, new StringBuilder(
-                            destFolder.getFullName()).append(separator).toString()));
-                        session.setParameter(param, mbox);
-                    }
-                    mboxEnabled = mbox.booleanValue();
-                }
+                final boolean mboxEnabled =
+                    MBoxEnabledCache.isMBoxEnabled(imapConfig.getImapServerSocketAddress(), destFolder, new StringBuilder(
+                        destFolder.getFullName()).append(separator).toString());
                 if (!checkFolderNameValidity(newName, separator, mboxEnabled)) {
                     throw IMAPException.create(IMAPException.Code.INVALID_FOLDER_NAME, imapConfig, session, Character.valueOf(separator));
                 }
@@ -833,15 +823,14 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                     }
                     tmp.append(newName);
                     renameFolder = (IMAPFolder) imapStore.getFolder(tmp.toString());
-                    final String param = MailSessionParameterNames.getParamMBox(accountId);
-                    // Check for MBox format
-                    Boolean mbox = (Boolean) session.getParameter(param);
-                    if (null == mbox) {
-                        mbox = Boolean.valueOf(IMAPCommandsCollection.supportsFolderType(par, FOLDER_TYPE, new StringBuilder(
-                            par.getFullName()).append(separator).toString()));
-                        session.setParameter(param, mbox);
-                    }
-                    mboxEnabled = mbox.booleanValue();
+                    /*
+                     * Check for MBox
+                     */
+                    mboxEnabled =
+                        MBoxEnabledCache.isMBoxEnabled(
+                            imapConfig.getImapServerSocketAddress(),
+                            par,
+                            new StringBuilder(par.getFullName()).append(separator).toString());
                 }
                 if (renameFolder.exists()) {
                     throw IMAPException.create(IMAPException.Code.DUPLICATE_FOLDER, imapConfig, session, renameFolder.getFullName());
@@ -948,7 +937,8 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                                 session,
                                 updateMe.getFullName());
                         }
-                        if (!getACLExtension().canSetACL(RightsCache.getCachedRights(updateMe, true, session, accountId))) {
+                        final ACLExtension aclExtension = getACLExtension();
+                        if (!aclExtension.canSetACL(RightsCache.getCachedRights(updateMe, true, session, accountId))) {
                             throw IMAPException.create(IMAPException.Code.NO_ADMINISTER_ACCESS, imapConfig, session, updateMe.getFullName());
                         }
                         /*
@@ -960,7 +950,7 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                         {
                             boolean adminFound = false;
                             for (int i = 0; (i < newACLs.length) && !adminFound; i++) {
-                                if (getACLExtension().canSetACL(newACLs[i].getRights())) {
+                                if (aclExtension.canSetACL(newACLs[i].getRights())) {
                                     adminFound = true;
                                 }
                             }
@@ -1011,8 +1001,8 @@ public final class IMAPFolderStorage extends MailFolderStorage {
         /*
          * Delete moved folder if operation failed
          */
-        final IMAPFolder tmp = (IMAPFolder) imapStore.getFolder(new StringBuilder(destFolder.getFullName()).append(
-            destFolder.getSeparator()).append(name).toString());
+        final IMAPFolder tmp =
+            (IMAPFolder) imapStore.getFolder(new StringBuilder(destFolder.getFullName()).append(destFolder.getSeparator()).append(name).toString());
         if (tmp.exists()) {
             try {
                 tmp.delete(true);
@@ -1056,15 +1046,16 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                     final String name = deleteMe.getName();
                     int appendix = 1;
                     final StringBuilder sb = new StringBuilder();
-                    IMAPFolder newFolder = (IMAPFolder) imapStore.getFolder(sb.append(trashFolder.getFullName()).append(
-                        deleteMe.getSeparator()).append(name).toString());
+                    IMAPFolder newFolder =
+                        (IMAPFolder) imapStore.getFolder(sb.append(trashFolder.getFullName()).append(deleteMe.getSeparator()).append(name).toString());
                     while (newFolder.exists()) {
                         /*
                          * A folder of the same name already exists. Append appropriate appendix to folder name and check existence again.
                          */
                         sb.setLength(0);
-                        newFolder = (IMAPFolder) imapStore.getFolder(sb.append(trashFolder.getFullName()).append(deleteMe.getSeparator()).append(
-                            name).append('_').append(++appendix).toString());
+                        newFolder =
+                            (IMAPFolder) imapStore.getFolder(sb.append(trashFolder.getFullName()).append(deleteMe.getSeparator()).append(
+                                name).append('_').append(++appendix).toString());
                     }
                     try {
                         moveFolder(deleteMe, trashFolder, newFolder, false);
@@ -1129,7 +1120,9 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                     return;
                 }
                 String trashFullname = null;
-                final boolean backup = (!hardDelete && !UserSettingMailStorage.getInstance().getUserSettingMail(session.getUserId(), ctx).isHardDeleteMsgs() && !(f.getFullName().startsWith((trashFullname = getTrashFolder()))));
+                final boolean backup =
+                    (!hardDelete && !UserSettingMailStorage.getInstance().getUserSettingMail(session.getUserId(), ctx).isHardDeleteMsgs() && !(f.getFullName().startsWith((trashFullname =
+                        getTrashFolder()))));
                 if (backup) {
                     imapAccess.getMessageStorage().notifyIMAPFolderModification(trashFullname);
                 }
@@ -1528,10 +1521,11 @@ public final class IMAPFolderStorage extends MailFolderStorage {
         /*
          * Ensure that owner still holds full rights
          */
-        final String ownerACLName = Entity2ACL.getInstance(imapConfig).getACLName(
-            session.getUserId(),
-            ctx,
-            IMAPFolderConverter.getEntity2AclArgs(session, defaultFolder, imapConfig));
+        final String ownerACLName =
+            Entity2ACL.getInstance(imapConfig).getACLName(
+                session.getUserId(),
+                ctx,
+                IMAPFolderConverter.getEntity2AclArgs(session, defaultFolder, imapConfig));
         for (int i = 0; i < newACLs.length; i++) {
             if (newACLs[i].getName().equals(ownerACLName) && newACLs[i].getRights().contains(FULL_RIGHTS)) {
                 return true;
@@ -1774,20 +1768,18 @@ public final class IMAPFolderStorage extends MailFolderStorage {
 
     private static boolean isKnownEntity(final String entity, final Entity2ACL entity2ACL, final Context ctx, final Entity2ACLArgs args) {
         try {
-            return entity2ACL.getEntityID(entity, ctx, args)[0] != -1;
+            return !UserGroupID.NULL.equals(entity2ACL.getEntityID(entity, ctx, args));
         } catch (final AbstractOXException e) {
             return false;
         }
     }
 
-    private static final Pattern PAT_RIGHT_POST = Pattern.compile("p|P");
-
     private boolean equals(final ACL[] oldACLs, final Map<String, ACL> newACLs, final Entity2ACL entity2ACL, final Entity2ACLArgs args) {
         int examined = 0;
         for (final ACL oldACL : oldACLs) {
-            final String aclName = oldACL.getName();
-            if (isKnownEntity(aclName, entity2ACL, ctx, args)) {
-                final ACL newACL = newACLs.get(aclName);
+            final String oldName = oldACL.getName();
+            if (isKnownEntity(oldName, entity2ACL, ctx, args)) {
+                final ACL newACL = newACLs.get(oldName/* .toLowerCase(Locale.ENGLISH) */);
                 if (null == newACL) {
                     // No corresponding entity in new ACLs
                     return false;
@@ -1795,8 +1787,7 @@ public final class IMAPFolderStorage extends MailFolderStorage {
                 // Remember number of corresponding entities
                 examined++;
                 // Check ACLS' rights ignoring POST right
-                if (!PAT_RIGHT_POST.matcher(oldACL.getRights().toString()).replaceFirst("").equals(
-                    PAT_RIGHT_POST.matcher(newACL.getRights().toString()).replaceFirst(""))) {
+                if (!equalRights(oldACL.getRights().toString(), newACL.getRights().toString(), true)) {
                     return false;
                 }
             }
@@ -1804,10 +1795,40 @@ public final class IMAPFolderStorage extends MailFolderStorage {
         return (examined == newACLs.size());
     }
 
+    private static String stripPOSTRight(final String rights) {
+        final StringBuilder sb = new StringBuilder(rights.length());
+        final char[] chars = rights.toCharArray();
+        for (int i = 0; i < chars.length; i++) {
+            final char c = chars[i];
+            if ('p' != c && 'P' != c) {
+                sb.append(c);
+            }
+        }
+        return sb.toString();
+    }
+
+    private static boolean equalRights(final String rights1, final String rights2, final boolean ignorePOST) {
+        final char[] r1;
+        final char[] r2;
+        if (ignorePOST) {
+            r1 = stripPOSTRight(rights1).toCharArray();
+            r2 = stripPOSTRight(rights2).toCharArray();
+        } else {
+            r1 = rights1.toCharArray();
+            r2 = rights2.toCharArray();
+        }
+        if (r1.length != r2.length) {
+            return false;
+        }
+        Arrays.sort(r1);
+        Arrays.sort(r2);
+        return Arrays.equals(r1, r2);
+    }
+
     private static Map<String, ACL> acl2map(final ACL[] acls) {
         final Map<String, ACL> m = new HashMap<String, ACL>(acls.length);
         for (final ACL acl : acls) {
-            m.put(acl.getName(), acl);
+            m.put(acl.getName()/* .toLowerCase(Locale.ENGLISH) */, acl);
         }
         return m;
     }
