@@ -640,95 +640,8 @@ public final class FetchIMAPCommand extends AbstractIMAPCommand<Message[]> {
 
     private static final class HeaderFetchItemHandler implements FetchItemHandler {
 
-        // private static interface HeaderHandler {
-        //
-        // /**
-        // * Handles given header value and applies it to given message.
-        // *
-        // * @param hdrValue The header value
-        // * @param msg The message to apply to
-        // * @throws MessagingException If a messaging error occurs
-        // * @throws MailException If a mail error occurs
-        // */
-        // public void handleHeader(String hdrValue, ExtendedMimeMessage msg) throws MessagingException, MailException;
-        // }
-
-        // static final MailDateFormat mailDateFormat = new MailDateFormat();
-
-        // private final Map<String, HeaderHandler> hdrHandlers;
-
         public HeaderFetchItemHandler() {
             super();
-            /*-
-             * 
-            this.hdrHandlers = new HashMap<String, HeaderHandler>();
-            hdrHandlers.put(MessageHeaders.HDR_FROM, new HeaderHandler() {
-
-                public void handleHeader(final String hdrValue, final ExtendedMimeMessage msg) throws MessagingException {
-                    try {
-                        msg.addFrom(InternetAddress.parse(hdrValue, false));
-                    } catch (final AddressException e) {
-                        msg.setHeader(MessageHeaders.HDR_FROM, hdrValue);
-                    }
-                }
-            });
-            hdrHandlers.put(MessageHeaders.HDR_TO, new HeaderHandler() {
-
-                public void handleHeader(final String hdrValue, final ExtendedMimeMessage msg) throws MessagingException {
-                    try {
-                        msg.setRecipients(RecipientType.TO, InternetAddress.parse(hdrValue, false));
-                    } catch (final AddressException e) {
-                        msg.setHeader(MessageHeaders.HDR_TO, hdrValue);
-                    }
-                }
-            });
-            hdrHandlers.put(MessageHeaders.HDR_CC, new HeaderHandler() {
-
-                public void handleHeader(final String hdrValue, final ExtendedMimeMessage msg) throws MessagingException {
-                    try {
-                        msg.setRecipients(RecipientType.CC, InternetAddress.parse(hdrValue, false));
-                    } catch (final AddressException e) {
-                        msg.setHeader(MessageHeaders.HDR_CC, hdrValue);
-                    }
-                }
-            });
-            hdrHandlers.put(MessageHeaders.HDR_BCC, new HeaderHandler() {
-
-                public void handleHeader(final String hdrValue, final ExtendedMimeMessage msg) throws MessagingException {
-                    try {
-                        msg.setRecipients(RecipientType.BCC, InternetAddress.parse(hdrValue, false));
-                    } catch (final AddressException e) {
-                        msg.setHeader(MessageHeaders.HDR_BCC, hdrValue);
-                    }
-                }
-            });
-            hdrHandlers.put(MessageHeaders.HDR_REPLY_TO, new HeaderHandler() {
-
-                public void handleHeader(final String hdrValue, final ExtendedMimeMessage msg) throws MessagingException {
-                    try {
-                        msg.setReplyTo(InternetAddress.parse(hdrValue, true));
-                    } catch (final AddressException e) {
-                        msg.setHeader(MessageHeaders.HDR_REPLY_TO, hdrValue);
-                    }
-                }
-            });
-            hdrHandlers.put(MessageHeaders.HDR_SUBJECT, new HeaderHandler() {
-
-                public void handleHeader(final String hdrValue, final ExtendedMimeMessage msg) throws MessagingException {
-                    msg.setHeader(MessageHeaders.HDR_SUBJECT, hdrValue);
-                }
-            });
-            hdrHandlers.put(MessageHeaders.HDR_DATE, new HeaderHandler() {
-
-                public void handleHeader(final String hdrValue, final ExtendedMimeMessage msg) throws MessagingException {
-                    try {
-                        msg.setSentDate(mailDateFormat.parse(hdrValue));
-                    } catch (final ParseException e) {
-                        throw new MessagingException(e.getMessage(), e);
-                    }
-                }
-            });
-             */
         }
 
         public void handleItem(final Item item, final ExtendedMimeMessage msg, final org.apache.commons.logging.Log logger) throws MessagingException, MailException {
